﻿#region Using directives

using System;
using System.Collections;
using System.ComponentModel.Design;
using System.Collections.Generic;
using System.Text;
using System.Xml.Serialization;

#endregion

namespace MonsieurWJ.LibXMS2 {


	/// <summary>
	/// Phone Book Types
	/// </summary>
	public class PhoneBookType {

		/// <summary>
		/// Phone's Phone Book
		/// </summary>
		public const string Phone = "ME";

		/// <summary>
		/// SIM's Phone Book
		/// </summary>
		public const string SIM = "SM";

		/// <summary>
		/// Dialled Calls Phone Book
		/// </summary>
		public const string DialledCalls = "DC";

		/// <summary>
		/// Missed Calls
		/// </summary>
		public const string MissedCalls = "MC";

		/// <summary>
		/// Received Calls
		/// </summary>
		public const string ReceivedCalls = "RC";

		/// <summary>
		/// Fixed Dialling
		/// </summary>
		public const string FixedDialing = "FD";

		/// <summary>
		/// Uninstantiable
		/// </summary>
		private PhoneBookType() { }
	}

	/// <summary>
	/// An Entry in the Phone Book
	/// </summary>
	[Serializable]
	public class PhoneBookEntry {
		private string name;
		private string phone;
		private byte toa;
		private int ord;
		private string bookType;

		/// <summary>
		/// Gets or sets name
		/// </summary>
		/// <value>Name</value>
		[XmlAttribute("Name", Namespace = "urn:MonsieurWJ:XMS:PhoneBook")]
		public string Name {
			get {
				return name;
			}
			set {
				name = value;
			}
		}

		/// <summary>
		/// Gets or sets phone no.
		/// </summary>
		/// <value>Phone No.</value>
		[XmlAttribute("Phone", Namespace = "urn:MonsieurWJ:XMS:PhoneBook")]
		public string Phone {
			get {
				return phone;
			}
			set {
				this.phone = value;
			}
		}

		/// <summary>
		/// Gets or sets type of address of phone no.
		/// </summary>
		/// <value>Type of Address</value>
		[XmlAttribute("ToA", Namespace = "urn:MonsieurWJ:XMS:PhoneBook")]
		public byte ToA {
			get {
				return toa;
			}
			set {
				this.toa = value;
			}
		}

		/// <summary>
		/// Gets location of phone number in the phone book
		/// </summary>
		/// <value>Location of phone number</value>
		[XmlAttribute("Location", Namespace = "urn:MonsieurWJ:XMS:PhoneBook")]
		public int Location {
			get {
				return ord;
			}
		}

		/// <summary>
		/// Gets phone book
		/// </summary>
		/// <value>Phone Book</value>
		/// <seealso cref="PhoneBookType"/>
		[XmlAttribute("PhoneBook", Namespace = "urn:MonsieurWJ:XMS:PhoneBook")]
		public string PhoneBook {
			get {
				return bookType;
			}
		}

		/// <summary>
		/// Creates a new instance of <see cref="PhoneBookEntry"/>
		/// </summary>
		/// <param name="ord">Location of the entry in the phone</param>
		/// <param name="name">Name in the contact</param>
		/// <param name="phone">Phone Number in the contact</param>
		/// <param name="at">Type of Address of phone number</param>
		/// <param name="pb">Phone Book</param>
		public PhoneBookEntry(int ord, string name, string phone, byte at, string pb) {
			this.name = name;
			this.phone = phone;
			this.toa = at;
			this.bookType = pb;
			this.ord = ord;
		}
	}

	/// <summary>
	/// Phone Book
	/// </summary>
	[Serializable]
	public class PhoneBook {

		private Hashtable hashName = null;
		private Hashtable hashPhone = null;
		private Hashtable hashOrd = null;
		private IList<PhoneBookEntry> list = null;

		[XmlElement("Entries", Namespace = "urn:MonsieurWJ:XMS:PhoneBook")]
		public PhoneBookEntry[] Entries {
			get {
				PhoneBookEntry[] a = new PhoneBookEntry[this.list.Count];
				this.list.CopyTo(a, 0);
				return a;
			}
		}

		/// <summary>
		/// Creates a new instance of <see cref="PhoneBook"/>
		/// </summary>
		public PhoneBook() {
			this.hashName = new Hashtable();
			this.hashPhone = new Hashtable();
			this.hashOrd = new Hashtable();
			this.list = new List<PhoneBookEntry>();
		}

		/// <summary>
		/// Adds a new <see cref="PhoneBookEntry"/> to this phonebook.
		/// </summary>
		/// <param name="a">An instance of the <see cref="PhoneBookEntry"/></param>
		public void Add(PhoneBookEntry a) {
			this.hashName[a.Name] = a;
			this.hashPhone[a.Phone] = a;
			this.hashOrd[a.Location] = a;
			this.list.Add(a);
		}

		public void Remove(string name) {
			PhoneBookEntry pb = (PhoneBookEntry)this.hashName[name];

			if(pb != null) {
				this.hashName.Remove(pb.Name);
				this.hashPhone.Remove(pb.Phone);
				this.hashOrd.Remove(pb.Location);
				this.list.Remove(pb);
			}
		}

		public PhoneBookEntry FindByName(string name) {
			return this.hashName[name] as PhoneBookEntry;
		}

		public PhoneBookEntry FindByPhone(string phone) {
			return this.hashPhone[phone] as PhoneBookEntry;
		}

		public PhoneBookEntry FindByLocation(int ord) {
			return this.hashPhone[ord] as PhoneBookEntry;
		}
	}
}
