﻿#region Using directives

using System;
using System.Collections;
using System.Text;

using MonsieurWJ.LibXMS2.Communicators.Basic;

#endregion

namespace MonsieurWJ.LibXMS2 {

	/// <summary>
	/// Communicator Factory
	/// </summary>
	public class CommunicatorFactory {

		private static CommunicatorFactory factory = null;
		private Hashtable communicators = null;

		static CommunicatorFactory() {
			CommunicatorFactory.factory = new CommunicatorFactory();
		}

		private CommunicatorFactory() {
			this.communicators = new Hashtable();
		}

		/// <summary>
		/// Gets default <see cref="CommunicatoryFactory"/> instance
		/// </summary>
		/// <value>Default <see cref="CommunicatorFactory"/> instance</value>
		public static CommunicatorFactory Default {
			get {
				return CommunicatorFactory.factory;
			}
		}

		/// <summary>
		/// Registers a communicator with this factory
		/// </summary>
		/// <param name="comm">Communicator to register</param>
		public void Register(ICommunicator comm) {
			this.communicators[comm.ModemID] = comm;
		}

		/// <summary>
		/// Unregisters a communicator from this factory
		/// </summary>
		/// <param name="modemID">Modem ID of the Communicator to unregister</param>
		public void Unregister(string modemID) {
			this.communicators.Remove(modemID);
		}

		/// <summary>
		/// Gets the communicator with the specific modem id
		/// </summary>
		/// <param name="id">Modem ID</param>
		/// <returns>An instance of <see cref="ICommunicator"/> if found, else
		/// null</returns>
		public ICommunicator this[string id] {
			get {
				return ((ICommunicator)this.communicators[id]);
			}
		}
	}
}
