﻿#region Using directives

using System;
using System.ComponentModel;
using System.Collections;
using generic = System.Collections.Generic;
using MonsieurWJ.LibXMS2.Internals;
#endregion

namespace MonsieurWJ.LibXMS2 {

	/// <summary>
	/// Represents a Modem
	/// </summary>
	public class Modem {

		private string name;
		private string port;
		private bool status;
		private int maxBaud;

		/// <summary>
		/// Constructs a new instance of <see cref="Modem"/>
		/// </summary>
		/// <param name="modem">A <see cref="POTSModem"/> instance</param>
		internal Modem(POTSModem modem) {
			if (modem != null) {
				this.name = modem.Caption;
				this.port = modem.AttachedTo;
				this.status = modem.Status.Equals("OK") ? true : false;
				this.maxBaud = (int)modem.MaxBaudRateToSerialPort;
			}
		}

		/// <summary>
		/// Gets name of the modem
		/// </summary>
		/// <value>Name of the modem</value>
		public string Name {
			get {
				return name;
			}
		}

		/// <summary>
		/// Gets communication port
		/// </summary>
		/// <value>Communication Port</value>
		public string CommPort {
			get {
				return port;
			}
		}

		/// <summary>
		/// Gets status of the modem
		/// </summary>
		/// <value>Status of the modem</value>
		public bool Status {
			get {
				return status;
			}
		}

		/// <summary>
		/// Gets Maximum Baud Rate to the Serial Port
		/// </summary>
		/// <value>Baud Rate</value>
		public int MaxBaudRate {
			get {
				return maxBaud;
			}
		}

		/// <summary>
		///  Gets list of modems attached to the system
		/// </summary>
		/// <returns>List of modems</returns>
		public static ICollection GetModems() {
			ICollection modems = POTSModem.GetInstances();
			ArrayList amodems = new ArrayList();
			int i = 0;
			foreach(POTSModem m in modems) {
				if (m.PortSubClass.Equals("\"02\""))
					amodems.Add(new Modem(m));
			}
//			Modem[] ms = new Modem[amodems.Count];
//			amodems.CopyTo(ms, 0);
			return amodems;
		}

		/// <summary>
		/// Creates a custom modem instance i.e. directly giving port name
		/// </summary>
		/// <param name="port">Name of the port</param>
		/// <param name="maxBaud">Maximum Baud Rate</param>
		/// <returns>A <see cref="Modem"/> instance</returns>
		public static Modem CreateModem(string port, int maxBaud) {
			Modem m = new Modem(null);
			m.maxBaud = maxBaud;
			m.port = port;
			m.name = port;
			m.status = true;

			return m;
		}

		/// <summary>
		/// Implements ToString() method
		/// </summary>
		/// <returns>Name of the modem</returns>
  		public override string ToString() {
			return this.name;
		}

	}
}
