﻿#region Using directives

using System;
using System.Text;
using System.IO.Ports;
using System.Threading;
using System.Collections.Generic;

#endregion

namespace MonsieurWJ.LibXMS2 {

	/// <summary>
	/// Modem Input Queue
	/// </summary>
	/// <remarks>
	/// This modem input queue is very easy to use. What you've to do initially is
	/// to bind the <see cref="ModemInputQueue.BytesRecieved"/> method to the
	/// <see cref="SerialPort.ReceivedEvent"/> as shown in the example. Example is
	/// in C#.
	/// </remarks>
	/// <example>
	///	this.port.ReceivedEvent += new SerialReceivedEventHandler(queue.BytesRecieved);
	/// </example>
	public class ModemInputQueue {

		// A Generic Queue :-)
		private Queue<string> q = null;
		private object o = new object();

		private bool waiting = false;

		/// <summary>
		/// Constructs a new instance of <see cref="ModemInputQueue"/>
		/// </summary>
		public ModemInputQueue() {
			this.q = new Queue<string>();
		}

		/// <summary>
		/// Gets Full Reply
		/// </summary>
		/// <returns>Reply from the Modem</returns>
		public string GetReply() {
			return this.GetReply(false);
		}

		/// <summary>
		/// Gets reply from the modem. This is a synchronous method.
		/// </summary>
		/// <param name="immediate">True if immediately available reply has to be returned,
		/// else it will return a reply that ends with OK or ERROR.</param>
		/// <returns>Reply from the Modem</returns>
		public string GetReply(bool immediate) {
			string reply = "";
			string z = null;

			do {
				if (this.q.Count == 0) {
					waiting = true;
					lock (o) {
						Monitor.Wait(o);
					}
					waiting = false;
				}
				z = this.q.Dequeue();
				reply += z;
				if (immediate)
					break;
			} while (z.IndexOf("OK") < 0 && z.IndexOf("ERROR") < 0 );

			return reply;
		}

		/// <summary>
		/// Event Handler for attaching attached to the <see cref="SerialPort.ReceivedEvent"/>
		/// event
		/// </summary>
		/// <param name="sender">Event Source</param>
		/// <param name="args">Event Arguments</param>
		/// <seealso cref="SerialPort.ReceivedEvent"/>
		public void BytesRecieved(object sender, SerialReceivedEventArgs args) {
			SerialPort port = sender as SerialPort;
			this.q.Enqueue(port.ReadExisting());
			if (waiting) {
				lock (o) {
					Monitor.Pulse(o);
				}
			}
		}
	}
}
