﻿#region Using directives

using System;
using System.IO;
using System.Text;
using System.IO.Ports;
using System.Threading;
using System.Collections;

using System.ComponentModel.Design;

using MonsieurWJ.LibXMS2.Communicators.Basic;

#endregion

namespace MonsieurWJ.LibXMS2 {

	/// <summary>
	/// eXaMS
	/// </summary>
    public sealed class XMS : IDisposable {

		private Modem modem = null;
		private ICommunicator communikator = null;

		/// <summary>
		/// Gets or sets modem for communication
		/// </summary>
		/// <value>Modem</value>
		public Modem Modem {
			set {
				if (value == null)
					throw new ArgumentNullException();
				this.modem = value;
			}
			get {
				return this.modem;
			}
		}

		/// <summary>
		/// Gets the communicator
		/// </summary>
		/// <returns>An instance of <see cref="ICommunicator"/></returns>
		public ICommunicator GetCommunicator() {
			return this.communikator;
		}

		/// <summary>
		/// Static Constructor
		/// </summary>
		static XMS() {
			CommunicatorFactory fac = CommunicatorFactory.Default;
			fac.Register(new MonsieurWJ.LibXMS2.Communicators.Nokia.Kommunikator());
		}

		/// <summary>
		/// Constructs a new instance of eXaMS
		/// </summary>
		public XMS() {
		}

		/// <summary>
		/// Starts communication
		/// </summary>
		public void Initialize() {
			SerialPort port = null;
			if (this.modem == null)
				throw new ModemException(ModemErrorType.NoModem, new ArgumentNullException());
			try {
				port = new SerialPort(modem.CommPort, modem.MaxBaudRate );
				port.ReadTimeout = 1000;
				port.WriteTimeout = 1000;
				port.ReadBufferSize = 16384;
				port.Open();
				port.Write("ATI\r\n");
				Thread.Sleep(1000);
				string x = Utility.GetProvider(port.ReadExisting(),"\r\n");
				if (x == null) {
					port.Close();
					return;// "Error [No Provider found]";
				}
				CommunicatorFactory factory = CommunicatorFactory.Default;
				this.communikator = factory[x];
				if(this.communikator == null)
					throw new ModemException(ModemErrorType.NoCommunicatorFound, 
							new Exception("No communicator found for \""+x+"\" modem"));
				XMS xms = this;
				this.communikator.Initialize(ref port);
			} catch (InvalidOperationException e) {
				throw new ModemException(ModemErrorType.AlreadyInUse, e);
			} catch (IOException e) {
				throw new ModemException(ModemErrorType.InvalidState, e);
			} catch (UnauthorizedAccessException e) {
				throw new ModemException(ModemErrorType.AccessDenied, e);
			} catch (ArgumentException e) {
				throw e;
			}
		}

		#region IDisposable Members

		public void Dispose() {
			if (this.communikator != null) {
				this.communikator.Dispose();
				this.communikator = null;
			}
		}

		#endregion
	}

}
