﻿#region Using directives

using System;
using System.Collections.Generic;
using System.Text;

using NUnit.Core;
using NUnit.Framework;

using MonsieurWJ.LibXMS2;

#endregion

namespace MonsieurWJ.LibXMS2.NTests {

	[TestFixture(Description = "GSM Encoding Test")]
	public class GSMTest {

		private GsmEncoding gsm = null;

		[Test(Description = "Tests GetByteCount method")]
		public void TestGetByteCount() {
			char[] s = "hellohello".ToCharArray();
			char[] t = "bye".ToCharArray();
			Assert.AreEqual(9, gsm.GetByteCount(s, 0, s.Length));
			Assert.AreEqual(3, gsm.GetByteCount(t, 0, t.Length));
		}

		[Test(Description = "Tests GetBytes method")]
		public void TestGetBytes() {
			char[] sz = "hellohello".ToCharArray();
			byte[] o = new byte[] {
				0xE8, 0x32, 0x9B, 0xFD,
				0x46, 0x97, 0xD9, 0xEC, 0x37
			};
			char[] sz2 = "hellohello{~}".ToCharArray();
			byte[] o2 = new byte[] {
				0xE8, 0x32, 0x9B, 0xFD ,0x46,
				0x97, 0xD9, 0xEC, 0xF7 ,0x06,
				0xB5, 0xE9, 0x6D, 0x52
			};

			byte[] e = gsm.GetBytes(sz);
			Assert.AreEqual(o.Length, e.Length);
			for (int i = 0; i < e.Length; i++)
				Assert.AreEqual(o[i], e[i]);

			e = gsm.GetBytes(sz2);
			Assert.AreEqual(o2.Length, e.Length);
			for (int i = 0; i < o2.Length; i++)
				Assert.AreEqual(o2[i], e[i]);

//			Console.WriteLine("Output: "+GsmEncoding.GSM.GetString(GsmEncoding.GSM.GetBytes("$$$")));
		}

		[Test(Description = "Tests GetCharCount method")]
		public void TestGetCharCount() {
			byte[] o = new byte[] {
				0xE8, 0x32, 0x9B, 0xFD,
				0x46, 0x97, 0xD9, 0xEC, 0x37
			};
			byte[] o2 = new byte[] {
				0xE8, 0x32, 0x9B, 0xFD ,0x46,
				0x97, 0xD9, 0xEC, 0xF7 ,0x06,
				0xB5, 0xE9, 0x6D, 0x52
			};
			Assert.AreEqual(10, gsm.GetCharCount(o));
			Assert.AreEqual(16, gsm.GetCharCount(o2));
		}

		[Test(Description = "Tests GetChars method")]
		public void TestGetChars() {
			byte[] o = new byte[] {
				0xE8, 0x32, 0x9B, 0xFD,
				0x46, 0x97, 0xD9, 0xEC, 0x37
			};
			byte[] h = new byte[] {
				0xC8, 0x3A, 0x7D, 0x8C, 0x06
			};
			byte[] i = new byte[] {
				0xE8, 0x32, 0x9B, 0xFD ,0x46,
				0x97, 0xD9, 0xEC, 0xF7 ,0x06,
				0xB5, 0xE9, 0x6D, 0x52
			};
			byte[] j = new byte[] {
				0xE8, 0x34
			};


			char[] ch = new char[gsm.GetCharCount(o)];
			gsm.GetChars(o, 0, o.Length, ch, 0);
			string sz = new string(ch);
			Assert.AreEqual("hellohello", sz);
			
			ch = new char[gsm.GetCharCount(h)];
			gsm.GetChars(h, 0, h.Length, ch, 0);
			sz = new string(ch);
			Assert.AreEqual("Hutch", sz);

			// This string requires escaping due to presence of sp. characters like '{','~','}'
			ch = new char[gsm.GetCharCount(i)];
			gsm.GetChars(i, 0, i.Length, ch, 0);
			sz = new string(ch);
			Assert.AreEqual("hellohello{~}", GsmHelper.Unescape(sz));

			ch = new char[gsm.GetCharCount(j)];
			gsm.GetChars(j, 0, j.Length, ch, 0);
			sz = new string(ch);
			Assert.AreEqual("hi", sz);
		}

		[Test(Description = "Tests GetMaxByteCount method")]
		public void TestGetMaxByteCount() {
			Assert.AreEqual(9, gsm.GetMaxByteCount(10));
		}

		[Test(Description = "Tests GetMaxCharCount method")]
		public void TestGetMaxCharCount() {
			Assert.AreEqual(10, gsm.GetMaxCharCount(9));
		}

		[TestFixtureSetUp]
		public void SetUp() {
			this.gsm = new GsmEncoding();
		}

		[TestFixtureTearDown]
		public void TearDown() {
			this.gsm = null;
		}
	}

	[TestFixture(Description = "GSM Helper Test")]
	public class GsmHelperTest {

		[Test(Description = "Tests Escape method")]
		public void TestEscape() {
			string sz_1 = "{~}";
			string osz_1 = "\x1b(\x1b=\x1b)";

			Assert.AreEqual(osz_1, GsmHelper.Escape(sz_1));
		}

		[Test(Description = "Tests Unescape method")]
		public void TestUnescape() {
			string osz_1 = "{~}";
			string sz_1 = "\x1b(\x1b=\x1b)";

			Assert.AreEqual(osz_1, GsmHelper.Unescape(sz_1));
		}
	}
}
