﻿#region Using directives

using System;
using System.Collections.Generic;
using System.Text;

using NUnit.Core;
using NUnit.Framework;

using MonsieurWJ.LibXMS2;

#endregion

namespace MonsieurWJ.LibXMS2.NTests {

	[TestFixture(Description = "HEX Encoding Test")]
	public class HexTest {

		private HexEncoding hex = null;

		[TestFixtureSetUp]
		public void SetUp() {
			this.hex = new HexEncoding();
		}

		[TestFixtureTearDown]
		public void TearDown() {
			this.hex = null;
		}

		[Test(Description = "Tests GetByteCount method")]
		public void TestGetByteCount() {
			char[] s = "9A10C9A0B9A000".ToCharArray();
			char[] t = "9A10FFC279A0B9A000".ToCharArray();
			Assert.AreEqual(7, hex.GetByteCount(s, 0, s.Length));
			Assert.AreEqual(9, hex.GetByteCount(t, 0, t.Length));
		}

		[Test(Description = "Tests GetBytes method")]
		public void TestGetBytes() {
			char[] sz = "9A10C9A0B9A00090".ToCharArray();
			byte[] o = new byte[] {
				0x9A, 0x10, 0xC9, 0xA0,
				0xB9, 0xA0, 0x00, 0x90
			};
			byte[] e = hex.GetBytes(sz);
			Assert.AreEqual(o.Length, e.Length);
			for (int i = 0; i < e.Length; i++)
				Assert.AreEqual(o[i], e[i], "at index: " + i);
		}

		[Test(Description = "Tests GetCharCount method")]
		public void TestGetCharCount() {
			byte[] o = new byte[] {
				0x9A, 0x10, 0xC9, 0xA0,
				0xB9, 0xA0, 0x00
			};
			byte[] p = new byte[] {
				0x9C, 0x10, 0xC9, 0xA0,
				0xB9, 0xA0
			};
			Assert.AreEqual(14, hex.GetCharCount(o));
			Assert.AreEqual(12, hex.GetCharCount(p));
		}

		[Test(Description = "Tests GetChars method")]
		public void TestGetChars() {
			byte[] o = new byte[] {
				0x9A, 0x10, 0xC9, 0xA0,
				0xB9, 0xA0, 0x00
			};
			char[] ch = new char[hex.GetCharCount(o)];
			hex.GetChars(o, 0, o.Length, ch, 0);
			string sz = new string(ch);
			Assert.AreEqual("9A10C9A0B9A000".ToLower(), sz.ToLower());
		}

		[Test(Description = "Tests GetMaxByteCount method")]
		public void TestGetMaxByteCount() {
			Assert.AreEqual(5, hex.GetMaxByteCount(10));
			Assert.AreEqual(10, hex.GetMaxByteCount(20));
			Assert.AreEqual(11, hex.GetMaxByteCount(22));
		}

		[Test(Description = "Tests GetMaxCharCount method")]
		public void TestGetMaxCharCount() {
			Assert.AreEqual(22, hex.GetMaxCharCount(11));
			Assert.AreEqual(10, hex.GetMaxCharCount(05));
			Assert.AreEqual(20, hex.GetMaxCharCount(10));
		}

	}
}
