﻿#region Using directives

using System;
using System.Collections.Generic;
using System.Text;

using NUnit.Core;
using NUnit.Framework;

using MonsieurWJ.LibXMS2;

#endregion

namespace MonsieurWJ.LibXMS2.NTests {

	[TestFixture(Description = "SMS SUBMIT Packet Test")]
	public class SmsSubmitPacketTest {

		private SmsSubmitPacket pkt = null;

		[Test(Description = "Tests WantsDeliveryReport property")]
		public void TestWantsDeliveryReport() {
			pkt.WantsDeliveryReport = true;
			Assert.AreEqual(true, pkt.WantsDeliveryReport);

			pkt.WantsDeliveryReport = false;
			Assert.AreEqual(false, pkt.WantsDeliveryReport);
		}

		[Test(Description = "Tests ReplyPathExists property")]
		public void TestReplyPathExists() {
			pkt.ReplyPathExists = true;
			Assert.AreEqual(true, pkt.ReplyPathExists);

			pkt.ReplyPathExists = false;
			Assert.AreEqual(false, pkt.ReplyPathExists);
		}

		[Test(Description = "Tests MessageEncoding property")]
		public void TestMessageEncoding() {
			pkt.MessageEncoding = EncodingType.GSM;
			Assert.AreEqual(EncodingType.GSM, pkt.MessageEncoding);

			pkt.MessageEncoding = EncodingType.OCTET;
			Assert.AreEqual(EncodingType.OCTET, pkt.MessageEncoding);
			
			pkt.MessageEncoding = EncodingType.UCS2;
			Assert.AreEqual(EncodingType.UCS2, pkt.MessageEncoding);

			pkt.MessageEncoding = EncodingType.Reserved;
			Assert.AreEqual(EncodingType.Reserved, pkt.MessageEncoding);
		}

		[Test(Description = "Tests MessageClass property")]
		public void TestMessageClass() {
			pkt.MessageClass = MessageClass.Class0;
			Assert.AreEqual(MessageClass.Class0, pkt.MessageClass);

			pkt.MessageClass = MessageClass.Class1;
			Assert.AreEqual(MessageClass.Class1, pkt.MessageClass);

			pkt.MessageClass = MessageClass.Class2;
			Assert.AreEqual(MessageClass.Class2, pkt.MessageClass);

			pkt.MessageClass = MessageClass.Class3;
			Assert.AreEqual(MessageClass.Class3, pkt.MessageClass);
		}

		[Test(Description = "Tests RejectDuplicates property")]
		public void TestRejectDuplicates() {
			pkt.RejectDuplicates = true;
			Assert.AreEqual(true, pkt.RejectDuplicates);

			pkt.RejectDuplicates = false;
			Assert.AreEqual(false, pkt.RejectDuplicates);
		}

		[Test(Description = "Tests UserDataHeaderExists property")]
		public void TestUserDataHeaderExists() {
			pkt.UserDataHeaderExists = true;
			Assert.AreEqual(true, pkt.UserDataHeaderExists);

			pkt.UserDataHeaderExists = false;
			Assert.AreEqual(false, pkt.UserDataHeaderExists);
		}

		[Test(Description = "Tests Validity property")]
		public void TestValidity() {
			pkt.Validity = ValidityType.NotPresent;
			Assert.AreEqual(ValidityType.NotPresent, pkt.Validity);

			pkt.Validity = ValidityType.Absolute;
			Assert.AreEqual(ValidityType.Absolute, pkt.Validity);

			pkt.Validity = ValidityType.Enhanced;
			Assert.AreEqual(ValidityType.Enhanced, pkt.Validity);
			
			pkt.Validity = ValidityType.Relative;
			Assert.AreEqual(ValidityType.Relative, pkt.Validity);
		}

		[Test(Description = "Tests DestinationAddress property")]
		public void TestDestinationAddress() {
			pkt.DestinationAddress_ToA = (byte)AddressType.International;
			pkt.DestinationAddress = "+919811618974";
			Assert.AreEqual("+919811618974", pkt.DestinationAddress);

			pkt.DestinationAddress_ToA = (byte)AddressType.GSM;
			pkt.DestinationAddress = "Thales";
			Assert.AreEqual("Thales", pkt.DestinationAddress);

			pkt.DestinationAddress_ToA = (byte)AddressType.National;
			pkt.DestinationAddress = "935786879898";
			Assert.AreEqual("935786879898", pkt.DestinationAddress);
		}

		[Test(Description = "Tests SMSC property")]
		public void TestSMSC() {
			pkt.SMSC_ToA = (byte)AddressType.GSM;
			pkt.SMSC = "Hutch";
			Assert.AreEqual("Hutch", pkt.SMSC);

			pkt.SMSC_ToA = (byte)AddressType.International;
			pkt.SMSC = "919811080898";
			Assert.AreEqual("+919811080898", pkt.SMSC);
		}

		[Test(Description = "Tests MessageText property")]
		public void TestMessageText() {
			this.pkt.MessageEncoding = EncodingType.GSM;
			this.pkt.MessageText = "Wah Java !!";
			Assert.AreEqual("Wah Java !!", this.pkt.MessageText);
			Assert.AreEqual(11, this.pkt.MessageLength);

			this.pkt.MessageEncoding = EncodingType.GSM;
			this.pkt.MessageText = "hellohello";
			Assert.AreEqual("hellohello", this.pkt.MessageText);
			Assert.AreEqual(10, this.pkt.MessageLength);

			this.pkt.MessageEncoding = EncodingType.OCTET;
			Assert.AreEqual(10, this.pkt.MessageLength);

			this.pkt.MessageEncoding = EncodingType.UCS2;
			Assert.AreEqual(20, this.pkt.MessageLength);

			this.pkt.MessageEncoding = EncodingType.UCS2;
			this.pkt.MessageText = "\x92a\x94d\x930\x936\x93E\x928\x94d\x924";
			Assert.AreEqual("प्रशान्त", this.pkt.MessageText);
			Assert.AreEqual(16, this.pkt.MessageLength);

			this.pkt.MessageEncoding = EncodingType.Reserved;
			Assert.AreEqual(null, this.pkt.MessageText);
		}

		[Test(Description = "Tests GetBytes method")]
		public void TestGetBytes() {
			string exp =
				"00B1000C911989197268270018AA8C092A094D09300923093E092E00200938094C0930092D002E00" +
				"4E0045005400200076002E0032002E0030002E00340030003600300037002C000D000A0048006F00" +
				"77002700730020095B093F0928094D09260917094000200067006F0069006E0067000D000A093509" +
				"3E09390020004A00610076006100200021002100200076002E0031002E0035002E0030";

			this.pkt.SMSC = null;
			this.pkt.ReplyPathExists = true;
			this.pkt.WantsDeliveryReport = true;
			this.pkt.UserDataHeaderExists = false;
			this.pkt.RejectDuplicates = false;
			this.pkt.DestinationAddress_ToA = (byte)AddressType.International;
			this.pkt.DestinationAddress = "919891278672";
			this.pkt.Validity = ValidityType.Relative;
			this.pkt.Expires = new TimeSpan(4, 0, 0, 0);
			this.pkt.MessageClass = MessageClass.Class0;
			this.pkt.MessageEncoding = EncodingType.UCS2;
			this.pkt.MessageText = "प्रणाम सौरभ.NET v.2.0.40607,\r\nHow's ज़िन्दगी going\r\nवाह Java !! v.1.5.0";

			byte[] b = this.pkt.GetBytes();
			Assert.AreEqual(exp, HexEncoding.HEX.GetString(b));
		}

		[Test(Description ="Tests Parse Method")]
		public void TestParse() {
			string sms =
				"00B1000C911989197268270019AA88092A094D09300923093E092E00200938094C0930092D002E00" +
				"4E00450054002C000D000A0053004D005300200069006E00200939093F0928094D09260940002009" +
				"64000D000A0020091C0935093E092C00200926094B00200964002000530075006300630065007300" +
				"7300210021000D000A0935093E09390020004A006100760061002000210021";
			byte[] bytes = HexEncoding.HEX.GetBytes(sms);

			SmsSubmitPacket pk = SmsSubmitPacket.Parse(bytes);
			Assert.AreEqual(MessageClass.Class1, pk.MessageClass);
			Assert.AreEqual(EncodingType.UCS2, pk.MessageEncoding);
			Assert.AreEqual("+919891278672", pk.DestinationAddress);
			Assert.AreEqual(true, pk.WantsDeliveryReport);
			Assert.AreEqual(true, pk.ReplyPathExists);
			Assert.AreEqual(false, pk.RejectDuplicates);
			Assert.AreEqual(false, pk.UserDataHeaderExists);
			Assert.AreEqual(ValidityType.Relative, pk.Validity);
			Assert.AreEqual(new TimeSpan(4, 0, 0, 0), pk.Expires);
			Assert.AreEqual("प्रणाम सौरभ.NET,\r\nSMS in हिन्दी ।\r\n जवाब दो । Success!!\r\nवाह Java !!",
				pk.MessageText);
		}

		[TestFixtureSetUp]
		public void SetUp() {
			this.pkt = new SmsSubmitPacket();
		}

		[TestFixtureTearDown]
		public void TearDown() {
			this.pkt = null;
		}
	}
}
