﻿#region Using directives

using System;
using System.Collections.Generic;
using System.Text;

using NUnit.Core;
using NUnit.Framework;

using MonsieurWJ.LibXMS2;

#endregion

namespace MonsieurWJ.LibXMS2.NTests {

	[TestFixture(Description = "Utility Class Test")]
	public class UtilityTest {

		[Test(Description = "Tests BcdToDecimal method")]
		public void TestBcdToDecimal() {
			Assert.AreEqual(10, Utility.BcdToDecimal(0x10));
			Assert.AreEqual(20, Utility.BcdToDecimal(0x20));
			Assert.AreEqual(90, Utility.BcdToDecimal(0x90));
			Assert.AreEqual(16, Utility.BcdToDecimal(0x16));
			Assert.AreEqual(50, Utility.BcdToDecimal(0x5f));
			try {
				Assert.AreEqual(17, Utility.BcdToDecimal(0x18));
			} catch (AssertionException) {
				return;
			}
			Assert.Fail();
		}

		[Test(Description = "Tests DecimalToBcd method")]
		public void TestDecimalToBcd() {
			Assert.AreEqual(0x10, Utility.DecimalToBcd(10));
			Assert.AreEqual(0x99, Utility.DecimalToBcd(99));
			Assert.AreEqual(0x35, Utility.DecimalToBcd(35));
			Assert.AreEqual(0x49, Utility.DecimalToBcd(49));
			Assert.AreEqual(0x09, Utility.DecimalToBcd(9));
			try {
				Assert.AreEqual(0x17, Utility.DecimalToBcd(18));
			} catch (AssertionException) {
				return;
			}
			Assert.Fail();
		}

		[Test(Description = "Tests Swap method")]
		public void TestSwap() {
			byte[] b1 = new byte[] {
				0x10, 0x15, 0x8C, 0xB9, 0x1b, 0x2C, 0x32, 0x13,
				0x31, 0x45, 0x85, 0x45, 0x88, 0x09, 0x78, 0x04,
				0x00, 0x01
			};
			byte[] b2 = new byte[] {
				0x01, 0x51, 0xC8, 0x9B, 0xb1, 0xC2, 0x23, 0x31,
				0x13, 0x54, 0x58, 0x54, 0x88, 0x90, 0x87, 0x40,
				0x00, 0x10
			};
			Utility.Swap(b1, 0, b1.Length);
			Assert.AreEqual(b2.Length, b1.Length);
			for (int i = 0; i < b1.Length; i++)
				Assert.AreEqual(b2[i], b1[i]);
		}

		[Test(Description = "Tests BytesToPhone method")]
		public void TestBytesToPhone() {
			byte[] p1 = new byte[] {
				0x8C, 0xA3, 0xD7, 0xC8, 0x60
			};
			byte toa1 = (byte)(AddressType.GSM);
			byte len1 = 0xA;

			byte[] p2 = new byte[] {
				0x91, 0x98, 0x91, 0x03, 0x00, 0x99
			};
			byte toa2 = (byte)(AddressType.International);
			byte len2 = 0x7;

			byte[] p3 = new byte[] {
				0x01, 0x13, 0x31, 0x69, 0x64, 0x6F
			};
			byte toa3 = (byte)(AddressType.Generic);
			byte len3 = 0xB;

			Assert.AreEqual("Hutch", Utility.BytesToPhone(toa1, len1, 1, p1));
			Assert.AreEqual("+919891030099", Utility.BytesToPhone(toa2, len2, 1, p2));
			Assert.AreEqual("01133169646", Utility.BytesToPhone(toa3, len3, 0, p3));
		}

		[Test(Description = "Tests PhoneToBytes method")]
		public void TestPhoneToBytes() {
			byte[] eb1 = new byte[] {
				0x8C, 0xA3, 0xD7, 0xC8, 0x60
			};
			string p1 = "Hutch";
			byte toa1 = (byte)(AddressType.GSM);
			byte[] o1 = null;

			byte[] eb2 = new byte[] {
				0x01, 0x13, 0x31, 0x69, 0x64, 0x6F
			};
			string p2 = "01133169646";
			byte toa2 = (byte)(AddressType.Generic);
			byte[] o2 = null;

			byte[] eb3 = new byte[] {
				0x91, 0x98, 0x11, 0x61, 0x89, 0x74
			};
			string p3 = "+919811618974";
			byte toa3 = (byte)(AddressType.International);
			byte[] o3 = null;

			o1 = Utility.PhoneToBytes(p1, toa1, 0);
			o2 = Utility.PhoneToBytes(p2, toa2, 0);
			o3 = Utility.PhoneToBytes(p3, toa3, 0);

			Assert.AreEqual(eb1.Length, o1.Length);
			for (int i = 0; i < eb1.Length; i++)
				Assert.AreEqual(eb1[i], o1[i]);

			Assert.AreEqual(eb2.Length, o2.Length);
			for (int i = 0; i < eb2.Length; i++)
				Assert.AreEqual(eb2[i], o2[i]);
			
			Assert.AreEqual(eb3.Length, o3.Length);
			for (int i = 0; i < eb3.Length; i++)
				Assert.AreEqual(eb3[i], o3[i]);
		}

		[Test(Description = "Tests UtcToQuartersBcd method")]
		public void TestUtcToQuartersBcd() {
			TimeSpan ts1 = new TimeSpan(5, 30, 00);
			byte q1 = 0x22;

			TimeSpan ts2 = new TimeSpan(-5, -30, 00);
			byte q2 = 0xA2;

			Assert.AreEqual(q1, Utility.UtcToQuartersBcd(ts1));
			Assert.AreEqual(q2, Utility.UtcToQuartersBcd(ts2));
		}

		[Test(Description = "Tests QuartersBcdToUtc method")]
		public void TestQuartersBcdToUtc() {
			TimeSpan ts1 = new TimeSpan(5, 30, 00);
			byte q1 = 0x22;

			TimeSpan ts2 = new TimeSpan(-5, -30, 00);
			byte q2 = 0xA2;

			Assert.AreEqual(ts1, Utility.QuartersBcdToUtc(q1));
			Assert.AreEqual(ts2, Utility.QuartersBcdToUtc(q2));
		}

		[Test(Description = "Tests BytesToTimestamp method")]
		public void TestBytesToTimestamp() {
			byte[] scts1 = new byte[] {
				0x04, 0x07, 0x27, 0x12, 0x33, 0x17, 0x22
			};
			DateTime dt1 = new DateTime(2004, 7, 27, 12, 33, 17);

			byte[] scts2 = {
				0x04, 0x07, 0x28, 0x17, 0x14, 0x43, 0x22
			};
			DateTime dt2 = new DateTime(2004, 07, 28, 17, 14, 43);

			Assert.AreEqual(dt1, Utility.BytesToTimestamp(scts1));
			Assert.AreEqual(dt2, Utility.BytesToTimestamp(scts2));
		}

		[Test(Description = "Test TimestampToBytes method")]
		public void TestTimestampToBytes() {
			byte[] scts1 = new byte[] {
				0x04, 0x07, 0x27, 0x12, 0x33, 0x17, 0x00
			};
			DateTime dt1 = new DateTime(2004, 7, 27, 12, 33, 17, DateTimeKind.Utc);
			byte[] b1 = Utility.TimestampToBytes(dt1);

			byte[] scts2 = {
				0x04, 0x07, 0x28, 0x17, 0x14, 0x43, 0x22
			};
			DateTime dt2 = new DateTime(2004, 07, 28, 17, 14, 43, DateTimeKind.Local);
			byte[] b2 = Utility.TimestampToBytes(dt2);

			Assert.AreEqual(scts1.Length, b1.Length);
			for(int i = 0; i < b1.Length; i++)
				Assert.AreEqual(scts1[i], b1[i]);

			Assert.AreEqual(scts2.Length, b2.Length);
			for (int i = 0; i < b2.Length; i++)
				Assert.AreEqual(scts2[i], b2[i]);
		}

		[Test(Description = "Tests GetEncoding method")]
		public void TestGetEncoding() {
			byte b1 = 0x18; //  UCS2 Encoding
			EncodingType enc1 = EncodingType.UCS2;

			byte b2 = 0x11; // GSMEncoding
			EncodingType enc2 = EncodingType.GSM;

			byte b3 = 0xf5; // 8bit Encoding
			EncodingType enc3 = EncodingType.OCTET;

			Assert.AreEqual(enc1, Utility.GetEncoding(b1));
			Assert.AreEqual(enc2, Utility.GetEncoding(b2));
			Assert.AreEqual(enc3, Utility.GetEncoding(b3));
		}

		[Test(Description = "Tests GetMessageClass method")]
		public void TestGetMessageClass() {
			byte b1 = 0x18;
			MessageClass cls1 = MessageClass.Class0;

			byte b2 = 0x11; // GSMEncoding
			MessageClass cls2 = MessageClass.Class1;

			byte b3 = 0xf6; // 8bit Encoding
			MessageClass cls3 = MessageClass.Class2;


			Assert.AreEqual(cls1, Utility.GetMessageClass(b1));
			Assert.AreEqual(cls2, Utility.GetMessageClass(b2));
			Assert.AreEqual(cls3, Utility.GetMessageClass(b3));
		}

		[Test(Description = "Tests ByteToTimespan method")]
		public void TestByteToTimespan() {
			TimeSpan ts = new TimeSpan(4, 0, 0, 0);
			Assert.AreEqual(ts, Utility.ByteToTimespan((byte)0xAA));

			ts = new TimeSpan(0, 10, 10, 0);
			Assert.AreEqual(ts, Utility.ByteToTimespan((byte)121));

			ts = new TimeSpan(20, 0, 0, 0);
			Assert.AreEqual(ts, Utility.ByteToTimespan((byte)186));

			ts = new TimeSpan(35, 0, 0, 0);
			Assert.AreEqual(ts, Utility.ByteToTimespan((byte)197));
		}

		[Test(Description = "Tests TimespanToByte method")]
		public void TestTimespanToByte() {
			TimeSpan ts = new TimeSpan(4, 0, 0, 0);
			Assert.AreEqual((byte)0xAA, Utility.TimespanToByte(ts));

			ts = new TimeSpan(0, 10, 10, 0);
			Assert.AreEqual((byte)121, Utility.TimespanToByte(ts));

			ts = new TimeSpan(20, 0, 0, 0);
			Assert.AreEqual((byte)186, Utility.TimespanToByte(ts));

			ts = new TimeSpan(35, 0, 0, 0);
			Assert.AreEqual((byte)197, Utility.TimespanToByte(ts));

		}

		[Test(Description = "Tests IsStringUnicode method")]
		public void TestIsStringUnicode() {
			string text1 = "\x918\x930\x918\x930";
			bool o1 = true;

			string text2 = "Home";
			bool o2 = false;

			Assert.AreEqual(o1, Utility.IsStringUnicode(text1));
			Assert.AreEqual(o2, Utility.IsStringUnicode(text2));
		}
	}
}
