﻿#region Using directives

using System;
using System.Collections;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

using MonsieurWJ.LibXMS2;
using MonsieurWJ.LibXMS2.Communicators.Basic;

#endregion

namespace Test {
	partial class MainForm : Form {

		private XMS xms = null;
		private Modem currentModem = null;

		public MainForm() {
			InitializeComponent();
			this.InitStuff();
		}

		[STAThread]
		public static void Main(string[] args) {
			Application.Run(new MainForm());
		}

		private void InitStuff() {
			ICollection m = Modem.GetModems();
			foreach (Modem x in m) {
				ToolStripMenuItem btn = new ToolStripMenuItem();
				btn.AutoToolTip = true;
				btn.Text = x.ToString();
				btn.ToolTipText = "Select " + btn.Text;
				btn.Tag = x;
				btn.DisplayStyle = ToolStripItemDisplayStyle.ImageAndText;
				btn.Image = Resources.Phone;
				btn.Click += new EventHandler(btnModemsClicked);
				cmbModems.DropDownItems.Add(btn); //.Add(x);
			}
			this.txtMsg.ImeMode = ImeMode.On;
			this.txtMsg.Text = "";
		}

		private void btnStartClicked(object sender, EventArgs e)	{
			try {
				Modem mo = currentModem;
				if (xms == null)
					xms = new XMS();
				xms.Modem = mo;
				this.UseWaitCursor = true;
				xms.Initialize();
				ICommunicator ck = xms.GetCommunicator();
				string m = "";

				m += "<p><u>IMEI Number</u>: <b>" + ck.SerialNo + "</b><br/>";
				m += "<u>Model</u>: <b>" + ck.Model + "</b><br/>";
				m += "<u>Manufacturer</u>: <b>" + ck.Manufacturer + "</b><br/>";
				m += "<u>Revision</u>: <b>" + ck.Revision.Replace("\r\n", "<br/>")+"</b><br/>";
				m += "<u>Message Center</u>: <b>" + ck.GetMessageCenter() + "</b>" + "</p>";

				ie.Document.Body.InnerHtml = m;
				m = "";

				DateTime start = DateTime.Now;
				ck.SetMessageStore(MessageStore.Phone);
				ICollection msgs = ck.GetMessages(MessageFolder.All);
				ck.SetMessageStore(MessageStore.SIM);
				ICollection msgs2 = ck.GetMessages(MessageFolder.All);

				int i = 1;
				foreach (SMS p in msgs)
					ie.Document.Body.InnerHtml += "<p>" + (i++) + ", ID: <b>" + p.ID + "</b><br/>" + this.GetMsg(p) + "<hr/>";

				foreach (SMS p in msgs2)
					ie.Document.Body.InnerHtml += "<p>" + (i++) + ", ID: <b>" + p.ID + "</b><br/>" + this.GetMsg(p) + "<hr/>";

				m += "<p><u>Battery Charge:</u><b>" + ck.GetBatteryCharge() + "</b></p>";
				m += "<hr><p><u>Signal Strength:</u><b>" + ck.GetSignalStrength() + "</b></p>";

				m += GetAddressBook(ck,"ME");
				m += GetAddressBook(ck, "SM");
				
				ie.Document.Body.InnerHtml += m;
				m = "";

				m += GetAddressBook(ck, "MC");
				m += GetAddressBook(ck, "DC");
				DateTime end = DateTime.Now;
				m += "<hr/><p><u>Time Taken:</u><b>" + end.Subtract(start) + "</b></p>";
				ie.Document.Body.InnerHtml += m;
				m = "";
				// +919811009998, 145
			} catch(Exception err) {
				ie.Document.Body.InnerHtml += "<p>Exception: <span style=\"color: darkblue\">" +
					err.Message + "</span><br/><span style=\"color: red\">" +
					err.InnerException.Message + "</span></p><p>"+err.StackTrace+"</p>";
			} finally {
				this.UseWaitCursor = false;
			}
		}

		private string GetAddressBook(ICommunicator comm, string f) {
			comm.SetPhoneBook(f);
			PhoneBook pb = comm.GetContacts();
			string html = "<p>PhoneBook: " + f + "<hr/>";
			PhoneBookEntry[] pbs = pb.Entries;

			foreach (PhoneBookEntry pe in pbs) {
				html += "<u>S. No.: </u><b>" + pe.Location + "</b><br/>";
				html += "<u>Name: </u><b>" + pe.Name + "</b><br/>";
				html += "<u>Phone: </u><b>" + pe.Phone + "</b> [" + pe.ToA + "]<br/>";
			}
			html += "<hr/></p>";

			return html;
		}

		private string GetMsg(SMS s) {
			object o = s.Packet;
			if (o is SmsStatusReportPacket) {
				SmsStatusReportPacket p = o as SmsStatusReportPacket;
				return "<span style='color: red; font-weight: bold'>Delivery Report</span><br/><b>Timestamp:</b> <u>" +
					p.Timestamp + "</u><br/><b>Delivery Timestamp:</b> " +
					"<u>" + p.DeliveryTimestamp + "</u><br/><b>Addressee:</b> <u>" + p.OriginatingAddress + "</u><br/>" +
					"<b>SMSC:</b> <u>" + p.SMSC + "</u><br/><b>Is Delivered:</b> <u>" + p.Delivered + "</u></p>";
			} else if (o is SmsDeliverPacket) {
				SmsDeliverPacket p = o as SmsDeliverPacket;
				return "<span style='color: red; font-weight: bold'>SMS DELIVER</span><br/><b>Timestamp: </b><u>" +
				p.Timestamp + "</u><br/><b>Message Length:</b>" + p.MessageText.Length +
				"<br/><b>User Data Header Exists</b>:"+p.UserDataHeaderExists+"<br/><b>More Messages:</b>"+
				p.MoreMessages+"<br/><b>Reply Path Exists</b>:"+p.ReplyPathExists+"<br/><b>Wants Delivery Report</b>:"+
				p.WantsDeliveryReport+
				"<br/><b>Sender</b>:" + p.OriginatingAddress + "<br/><b>SMSC</b>:" + p.SMSC +
				"<br/><b>Message Class</b>:" + p.MessageClass + "<br/><b>Message Encoding</b>:" + p.MessageEncoding +
				"<br/><b>Message</b><br/>" + p.MessageText + "</p>";
			} else if (o is SmsSubmitPacket) {
				SmsSubmitPacket p = o as SmsSubmitPacket;
				return "<span style='color: red; font-weight: bold'>SMS SUBMIT</span><br/><b>Expires: </b><u>" +
					((p.Validity == ValidityType.NotPresent) ? "Not Present" : p.Expires) + "</u><br/><b>Message Length:</b>" + p.MessageText.Length +
				"<br/><b>Sender</b>:" + p.DestinationAddress + "<br/><b>SMSC</b>:" + p.SMSC +
				"<br/><b>Message Class</b>:" + p.MessageClass + "<br/><b>Message Encoding</b>:" + p.MessageEncoding +
				"<br/><b>Message</b><br/>" + p.MessageText + "</p>";
			} else
				return "<span style='color: red; font-weight: bold'>Not Recognized: " + o + "</span>";
		}

		private void formClosed(object sender, FormClosedEventArgs e) {
			if(this.xms != null)
				this.xms.Dispose();
		}

		private void btnModemsClicked(object sender, EventArgs e) {
			Modem m = null;
			if (sender is ToolStripComboBox) {
				ToolStripComboBox cmb = sender as ToolStripComboBox;
				m = Modem.CreateModem(cmb.Text, 9600);
			} else {
				ToolStripMenuItem btn = sender as ToolStripMenuItem;
				m = btn.Tag as Modem;
			}
			lblModem.Text = m.ToString();
			lblModem.ToolTipText = "Selected Modem: " + lblModem.Text;
			this.currentModem = m;
			ie.Document.Body.InnerHtml = "<p>Status: <b>" + m.Status + "</b></p>";
			if (m.Status)
				btnStart.Enabled = true;
			else
				btnStart.Enabled = false;
		}

		private void OnMsgSave(object sender, LinkLabelLinkClickedEventArgs e) {
			SmsSubmitPacket pkt = new SmsSubmitPacket();
			pkt.MessageClass = MessageClass.Class1;
			pkt.MessageEncoding = EncodingType.GSM;
			pkt.MessageText = txtMsg.Text;
			pkt.DestinationAddress_ToA = (byte)AddressType.Generic;
			pkt.DestinationAddress = "1111111111";
			pkt.Validity = ValidityType.Absolute;
			pkt.Expires = new DateTime(2004, 08, 30);

			ICommunicator ck = this.xms.GetCommunicator();
			ck.SetMessageStore(MessageStore.Phone);
			ck.Save(pkt);

//			ck.SetPhoneBook(PhoneBookType.Phone);
//			ck.DeleteContact(32);
//			ck.WriteContact(new PhoneBookEntry(32, "\x924\x928\x94d\x935\x940","+919811667217",(byte)145, "ME"));

			this.txtMsg.Text = "";
		}
	}
}